#include "MSP430F5529.h"
#include "HardwareConfig.h"
#include "LCD20x4.h"
#include "Types.h"

LCD20x4 SysLCD;

void LCD20x4::enable() {
	//	Enable the 3 LCD control lines
	LCD_CTRL_DIR	|=	(LCD_E | LCD_CD | LCD_WR);
	//	At start, CS should be 1 (disable), then the 2 are 0 in most cases.
	LCD_CTRL_OUT &= ~(LCD_E | LCD_WR | LCD_CD);
	//	Enable data out
	LCD_DATA_DIR = 0xFF;
	LCD_DATA_OUT = 0;
}

void LCD20x4::Pulse() {
	P6OUT |= 0x02;
	P6OUT &= ~0x02;
	P6OUT |= 0x02;
	P6OUT &= ~0x02;
}

void LCD20x4::Init() {
	enable();
	delay_ms(40);
	write_cmd(0x30);
	delay_ms(5);
	write_cmd(0x30);
	delay_ms(1);			//	Specs say 160us.
	write_cmd(0x30);
	delay_ms(1);			//	Specs say 160us.
	write_cmd(0x38);
	write_cmd(0x10);
	write_cmd(0x0C);
	write_cmd(0x06);
	write_cmd(LCD_CLEAR);
	delay_ms(2);			//	1.52 ms according to specs
}

void LCD20x4::write_cmd(uint8 c) {
	SET_WC();
	LCD_CTRL_OUT |= LCD_E;
	LCD_DATA_OUT = c;
	__delay_cycles(EN_DELAY);
	LCD_CTRL_OUT &= ~LCD_E;
	__delay_cycles(CMD_DELAY);
}

void LCD20x4::MoveTo(uint8 column, uint8 line) {
	uint8 dram_add = 0;
	switch(line) {
	case 0:
		dram_add = column;
		break;
	case 1:
		dram_add = 0x40 + column;
		break;
	case 2:
		dram_add = 0x14 + column;
		break;
	case 3:
		dram_add = 0x54 + column;
		break;
	}
	write_cmd(0x80 + dram_add);
}

void LCD20x4::Write(char c) {
	SET_WD();
	LCD_CTRL_OUT |= LCD_E;
	LCD_DATA_OUT = c;
	__delay_cycles(EN_DELAY);
	LCD_CTRL_OUT &= ~LCD_E;
	__delay_cycles(CMD_DELAY);
}

void LCD20x4::Write(char * str, uint8 len) {
	uint8 i;
	for(i = 0 ; i < len ; ++i) {
		if(str[i] == 0) return;
		Write(str[i]);
	}
}

//	NB: this method is intended to write 3 digit numbers only. Careful if you
//	use it for greater values, it has to be tuned.
void LCD20x4::WriteIntVal(int16 val) {
	uint8 m, c, d, u;
	uint8 sign;
	if(val < 0) {
		sign = '-';
		val = -val;
	}
	else {
		sign = ' ';
	}
	m = 0;
	c = 0;
	d = 0;
	u = 0;
	while(val >= 1000) {
		m++;
		val -= 1000;
	}	
	while(val >= 100) {
		c++;
		val -= 100;
	}
	while(val >= 10) {
		d++;
		val -= 10;
	}
	u = val;
	SysLCD.MoveTo(6, 2);
	Write(sign);	
	Write('0' + c);
	Write('0' + d);
	Write('0' + u);
}
